#!/usr/bin/env python3
# -*- coding: future_fstrings -*-
import os
import pathlib
import re
from typing import List, Dict

from migrationsummary.models.Summary import Summary
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
from migrationsummary.types import SQL


def get_website_name(site_id: int) -> str:
    """Gets the domainname of a website id

    Returns
    -------
    type
       str containing domain name

    """
    stmt = f"""SELECT domain FROM web_domain WHERE type LIKE 'vhost%'
               AND domain_id = {site_id};"""
    result = SQL.db_query(stmt).result
    return result[0]['domain'] if result else ""


def get_magento1_instances() -> List[str]:
    """Gets all Magento 1 Instances

    Returns
    -------
    type
       list containing domain names of Magento 1 Installations

    """
    m1instances = pathlib.Path('/var/www/clients/').glob(
        'client*/web*/web/**/Mage_Core_Modules-*.xml')

    m1domains = []
    try:
        for m1instance in m1instances:
            # instance_path = m1instance.__fspath__()
            instance_path = m1instance.as_posix()
            match = re.search(r'(?:web)([0-9]+)(?:/web)', instance_path)
            if match is None:
                continue

            site_id = match.group(1)
            m1domains.append(get_website_name(site_id))

        # print(m1domains)
        """
        ['meinemagentodomain.de']
        """
    except OSError as e:
        print(f'Warning: Could not check Magento 1 Instances, Error: {e}')
    finally:
        return m1domains


def get_magento2_instances() -> Dict[str, Dict[str, str]]:
    """Gets all Magento 2 Instances and their versions

    Returns
    -------
    type
       dictionary containing domain names of Magento 2 Installations
                  and their versions

    """
    m2domains = {}
    m2instances = pathlib.Path('/var/www/clients/').glob(
        'client*/web*/web/**/bin/magento')
    try:
        for m2instance in m2instances:
            # instance_path = m2instance.__fspath__()
            instance_path = m2instance.as_posix()
            match = re.search(
                (r'/var/www/clients/client[0-9]+/web([0-9]+)/web'
                 r'(?:(?!vendor/magento|dev/tests).)*?/bin/magento'),
                instance_path)
            if match is None:
                continue

            site_id = match.group(1)
            magento_path = match.group(0)[:-12]

            php_paths = pathlib.Path('/opt/').glob(
                f'php-*/etc/pool.d/web{site_id}.conf')
            php_version = ''
            for phpPath in php_paths:
                # match = re.search(r'php-([0-9.]+)', phpPath.__fspath__())
                match = re.search(r'php-([0-9.]+)', phpPath.as_posix())
                if match is None:
                    continue

                php_version = match.group(1)
                break

            m2domains[magento_path] = {'id': site_id,
                                       'name': get_website_name(site_id),
                                       'php_version': php_version,
                                       'magento_path': magento_path}
    except OSError as e:
        print(f'Warning: Could not check Magento 2 Instances, Error: {e}')

    for domain in m2domains:
        version = os.popen(
            (f"/opt/php-{m2domains[domain]['php_version']}/bin/php "
             f"{m2domains[domain]['magento_path']}/bin/magento "
             "--version")).read()
        m2domains[domain]['magento_version'] = version[12:]

    # print(m2domains)
    """
    {
      '/var/www/clients/client1/web1/web': {
        'id': '1',
        'name': 'meinemagento2domain.de',
        'php_version': '8.1',
        'magento_path': '/var/www/clients/client1/web1/web',
        'magentoVersion': 'Magento CLI 2.4.5-p2\n'
      },
      '/var/www/clients/client1/web1/web/staging': {
        'id': '1',
        'name': 'meinemagento2domain.de',
        'php_version': '8.1',
        'magento_path': '/var/www/clients/client1/web1/web/staging',
        'magentoVersion': 'Magento CLI 2.4.5-p2\n'
      }
    }
    """
    return m2domains


class MagentoCheck(Summary):

    def __init__(self):
        super().__init__(plugin_name='cmschecks',
                         template='cmschecks.j2')
        self.snippet_list = {}

    def summarize(self) -> None:
        tpl_init = TemplateRenderer(self.plugin_name, self.template)
        tpl_str = tpl_init.template_render()
        print(tpl_str.strip())

        m1_instances = get_magento1_instances()
        m2_instances = get_magento2_instances()

        out_str = ""
        for m1_instance in m1_instances:
            out_str += f"--> {m1_instance}\n"
        if out_str:
            title = "Magento 1 Installations"
            print(f"\n{title}:\n{'=' * len(title)}\n{out_str}")

        out_str = ""
        for m2_instance in m2_instances:
            out_str += f"--> Domain: {m2_instances[m2_instance]['name']}\n\t"
            out_str += f"Path: {m2_instances[m2_instance]['magento_path']}\n\t"
            out_str += f"Version: {m2_instances[m2_instance]['magento_version']}\n"
        if out_str:
            title = "Magento 2 Installations"
            print(f"\n{title}:\n{'=' * len(title)}\n{out_str}")


if __name__ == '__main__':
    # None
    foo = MagentoCheck()
    foo.summarize()
