# -*- coding: utf-8 -*-
from collections import defaultdict
from os.path import isfile
from migrationsummary.db import myisam
from migrationsummary.db import rowsize
from migrationsummary.models.Summary import Summary
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
from migrationsummary.types.SQL import SQL
import pymysql as my
from pymysql.cursors import DictCursor
import re
from typing import Text


class DbProperties:

    def __init__(self, db_type: str, db_version_str: str):
        self.type_info = db_type
        self.version_str = db_version_str

    @property
    def type_info(self) -> str:
        return self._db_type

    @type_info.setter
    def type_info(self, db_type: str) -> None:
        self._db_type = db_type

    @property
    def version_str(self) -> str:
        pattern = r'^(([0-9]{1,2}\.){2}\d+).*$'
        regex = re.compile(pattern=pattern)
        version_str = regex.match(self._db_version_str)
        if version_str.group(1):
            return version_str.group(1)
        return self._db_version_str

    @version_str.setter
    def version_str(self, db_version_str: str) -> None:
        self._db_version_str = db_version_str

    def version_is_deprecated(self) -> bool:
        version_eol = {
            'mysql': [
                {
                    'version': '5.7'
                }
            ],
            'mariadb': [
                {
                    'version': '10.11'
                }
            ],
            'percona': [
                {
                    'version': '5.7'
                }
            ]
        }
        if not self._db_type:
            raise ValueError('Error: Unrecognized database type')

        if self._db_type.lower().startswith('mysql' or 'percona'):
            return True
        for db_type, db_versions in version_eol.items():
            min_version = min(
                [db_version['version'] for db_version in db_versions])
            if (self._db_type.lower().startswith(db_type)
                    or self._db_type.lower() in db_type):
                if self.version_str < min_version:
                    return True
        return False


class Db(Summary):

    def __init__(self) -> None:
        super().__init__(plugin_name='database',
                         template='database_summary.j2')

    @property
    def db_conf(self) -> my.connections.Connection:
        sql_conf = '/etc/mysql/debian.cnf'
        if not isfile(sql_conf):
            raise Exception('Fatal: %s - No such file or directory'
                            % (sql_conf))
        try:
            return my.connect(read_default_file=sql_conf,
                              cursorclass=DictCursor)
        except my.err.OperationalError:
            raise Exception('Fatal: Unable to connect to database server.')

    @property
    def get_server_info(self) -> defaultdict or None:
        '''SQL Statement returns a dict like this (e.g. MariaDB):

        {
            'db_version': '10.3.28',
            'version_str': '10.3.28-MariaDB-1:10.3.28+maria~stretch',
            'db_type': 'mariadb.org binary distribution'
        }'''
        try:
            sql_stmt = """SELECT @@version_comment db_type, 
                                 @@version version_str;"""
            sql_res = SQL.db_query(sql_stmt=sql_stmt,
                                   db='',
                                   fetch_all=False)
            if not sql_res.result:
                return defaultdict(None)
            return defaultdict(Text, sql_res.result)
        except Exception:
            return None

    @property
    def db_info(self) -> DbProperties:
        return DbProperties(db_type=self.get_server_info['db_type'],
                            db_version_str=self.get_server_info['version_str'])

    def __summary_set_state(self) -> None:
        log_parser = LogParser()
        self.summary['type_state'] = log_parser.set_log_level(
            log_parser.INFO
            if self.db_info.type_info.lower().startswith('mariadb')
            else log_parser.ERROR)
        self.summary['state'] = log_parser.set_log_level(log_parser.OK)
        if re.search(r'(10\.([0-9]|10)\.|5\.7)', self.db_info.version_str):
            self.summary['state'] = log_parser.set_log_level(log_parser.WARN)
        if self.db_info.version_is_deprecated():
            self.summary['state'] = log_parser.set_log_level(log_parser.ERROR)

    def __summary_set_db_type(self) -> None:
        self.summary['db_type'] = self.db_info.type_info

    def __summary_set_db_version_str(self) -> None:
        self.summary['db_version_str'] = self.db_info.version_str

    def summarize(self) -> None:
        if self.get_server_info is not None:
            self.__summary_set_db_type()
            self.__summary_set_state()
            self.__summary_set_db_version_str()

            tpl_init = TemplateRenderer(self.plugin_name, self.template)
            tpl_str = tpl_init.template_render(**{'db_config': self.summary})
            print(tpl_str)
            print(myisam.summary())
            print(rowsize.summary())
        else:
            print("ERROR - Unable to connect to database server!")
