#!/usr/bin/env python3
# -*- coding: future_fstrings -*-

"""
This module is used to check for tables which exceed the max rowsize
"""
import re
from concurrent.futures import ProcessPoolExecutor
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.types.SQL import SQL


def process_database(database):
    """
    Retrieves a list of all tables which exceed the max rowsize
    """
    # print(f"checking database: {database}")
    problematictables = []
    sql_conn = SQL.db_conf(use_db=None, useunicode=False)
    sql_cursor = sql_conn.cursor()

    # Enable necessary settings
    sql_cursor.execute("SET innodb_strict_mode=1;")
    sql_cursor.execute("SET foreign_key_checks=0;")

    tempdatabase = f'{database}_temp3567'

    # Create temporary database
    sql_query = f"CREATE DATABASE {tempdatabase};"
    sql_cursor.execute(query=sql_query)
    if len(sql_cursor.fetchall()) > 0:
        raise Exception("Creating temporary database failed")

    sql_conn.select_db(tempdatabase)

    # Retrieve the list of tables in the current database
    sql_cursor.execute(f"SHOW TABLES FROM {database};")
    tables = [table[f"Tables_in_{database}"] for table in
              sql_cursor.fetchall()]

    for table in tables:
        table = table.decode("utf-8", "replace")
        # Retrieve the CREATE TABLE statement for the current table
        sql_cursor.execute(f"SHOW CREATE TABLE `{database}`.`{table}`;")

        # We are interested in "Create Table" and "Create View" entries,
        # so we need to do a search on all items to get the right entry
        search_key = "Create "
        # print(f"get create stmt result")
        createstmt = [val for key, val in sql_cursor.fetchall()[0].items()
                      if search_key in key][0].decode("utf-8", "replace")

        if "ROW_FORMAT" not in createstmt:
            # on new servers DYNAMIC is the default, so we can safely add it
            # if no other ROW_FORMAT is part of the create statement
            createstmt += " ROW_FORMAT=DYNAMIC"

        # print(f"replace tablename in create stmt")
        createstmt = re.sub(f"`{table}` \\(",
                            f"`{database}_{table}` (",
                            createstmt)

        try:
            # print(f"try creating: {createstmt}")
            sql_cursor.execute(createstmt)
            # print("created")

        except Exception as e:
            # print(f"error: {str(e)}")
            if "Row size too large" in str(e):
                problematictables.append(f"{database}.{table}")

    # print("trying to drop")
    sql_cursor.execute(f"DROP DATABASE {tempdatabase};")
    # print("dropped, trying to close")
    sql_conn.close()
    # print(f"tables: {problematictables}")

    return problematictables


def summary() -> str:
    log_parser = LogParser()
    level = log_parser.set_log_level(log_parser.OK)
    sum_str = ''
    try:
        sql_conn = SQL.db_conf(use_db=None)
        sql_cursor = sql_conn.cursor()

        problematictables = []

        # Retrieve the list of databases
        sql_query = f"""SHOW DATABASES
                WHERE `Database` NOT IN (
                    'sys',
                    'information_schema',
                    'mysql',
                    'performance_schema',
                    'dbispconfig',
                    'rainloop',
                    'phpmyadmin',
                    'grafana'
                );"""
        # print("get databases")

        sql_cursor.execute(query=sql_query)
        sql_conn.close()

        databases = [row['Database'] for row in sql_cursor.fetchall()]

        # Process databases in parallel
        with ProcessPoolExecutor(max_workers=1) as executor:
            for tables in executor.map(process_database, databases):
                # print(f"finished tables: {tables}")
                problematictables.extend(tables)
        # sequential:
        # for database in databases:
        #     problematictable = process_database(database)
        #     problematictables.extend(problematictable)

        # print(problematictables)
        if problematictables:
            level = log_parser.set_log_level(log_parser.ERROR)
            db_sum_str = f"[{level}] Found tables which exceed rowsize:\n"
            for item in problematictables:
                db_sum_str += f"    \\_ {item}\n"
            sum_str = db_sum_str
    except Exception as e:
        sum_str = "could not check row size of tables, please do so manually\n"
        sum_str += f"Error: {e}"

    sum_str = f'[{level}] No issues found\n' if sum_str == '' else sum_str

    sum_str = '--> \033[46mSubmodule: rowsize\033[0m\n' + sum_str

    return sum_str


if __name__ == '__main__':
    print(summary())
