# -*- coding: utf-8 -*-
from typing import Generator
from apt.cache import Cache
from collections import defaultdict
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.models.Summary import Summary
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
from migrationsummary.parsing.config.YamlParser import YamlParser
from os import path


class Debian(Summary):
    def __init__(self) -> None:
        super().__init__(plugin_name='debian', template='debian_summary.j2')
        self._dependency_list = set()
        self._package_list = set()
        self.summary = defaultdict(None)

    @property
    def dependency_list(self) -> set:
        return self._dependency_list

    @dependency_list.setter
    def dependency_list(self, dependency_list: set) -> None:
        self._dependency_list = dependency_list

    @property
    def package_list(self) -> set:
        return self._package_list

    @package_list.setter
    def package_list(self, package_list: set) -> None:
        self._package_list = package_list

    @property
    def yaml_pkg_conf(self) -> str:
        return path.join('%s/conf' % (self.module_path), 'apt_packages.yml')

    def __apt_get_installed(self) -> list:
        "Returns a defaultdict with installed packages and its dependencies"
        pkg_cache = Cache()
        pkg_list = []
        for pkg in pkg_cache:
            if (not pkg.essential and not pkg.is_auto_installed
                    and pkg.is_installed):
                pkg_dict = defaultdict(None)
                try:
                    deps_list = pkg.candidate.get_dependencies(
                            'PreDepends', 'Depends', 'Recommends')
                    pkg_dict['name'], pkg_dict['deps'] = pkg.name, [
                            item.name for dep in deps_list
                            for item in dep.or_dependencies if all(deps_list)
                            ]
                    pkg_list.append(pkg_dict)
                except AttributeError:
                    pkg_dict['name'], pkg_dict['deps'] = (
                            "\033[0;31m[Failed retrieving package]\033[0m"), ""
                    pkg_list.append(pkg_dict)
        return pkg_list

    def __dependency_list_append(self, dep_list: list) -> None:
        for dep in dep_list:
            if dep not in self._dependency_list:
                self._dependency_list.add(dep)

    def __package_list_append(self, pkg_name: str) -> None:
        if pkg_name not in self._package_list:
            self._package_list.add(pkg_name)

    def __filter_packages(self, package_list: set,
                          dependency_list: set) -> Generator:
        """Sort and filter packages out
        which are not included inside dependency list.
        These resulting packages are the ones which were manual installed."""
        for pkg_name in sorted(package_list):
            if pkg_name not in dependency_list:
                yield pkg_name

    def __yaml_retrieve_pkg(self) -> list:
        "Retrieve ideal packages from yaml config"
        parser = YamlParser(self.yaml_pkg_conf, 'r')
        yml_data = ''.join(parser.yaml_file.read())
        return parser.parse(yml_data)

    def __summary_set_pkg(self, pkg_list: set) -> None:
        "Append list of conflicting packages to summary"
        log_parser = LogParser()
        self.summary['pkg_variances'] = [
            defaultdict(
                None, {
                    key: val
                    for key, val in zip(('pkg_name', 'state'), (
                        pkg_name, log_parser.set_log_level(log_parser.WARN)))
                }) for pkg_name in pkg_list
        ]

    def __summary_set_conflict_flag(self) -> None:
        self.summary['has_conflicts'] = (
            True if self.summary['pkg_variances']
            else False
        )

    def __summary_set_state(self) -> None:
        log_parser = LogParser()
        self.summary['state'] = log_parser.set_log_level(
            log_parser.WARN if self.summary['pkg_variances'] else log_parser.OK
        )

    def summarize(self) -> None:
        pkg_list = self.__apt_get_installed()
        if all(pkg_list):
            for pkg_dict in pkg_list:
                self.__package_list_append(pkg_dict['name'])
                self.__dependency_list_append(pkg_dict['deps'])
        # Filter installed packages
        pkg_list = set()
        for pkg_name in self.__filter_packages(self._package_list,
                                               self._dependency_list):
            pkg_list.add(pkg_name)
        ideal_pkg_list = set(self.__yaml_retrieve_pkg())
        pkg_diff = pkg_list - ideal_pkg_list
        self.__summary_set_pkg(pkg_diff)
        self.__summary_set_conflict_flag()
        self.__summary_set_state()

        tpl_init = TemplateRenderer(self.plugin_name, self.template)
        tpl_str = tpl_init.template_render(**{'apt_config': self.summary})
        print(tpl_str)
