#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from collections import defaultdict
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.models.Summary import Summary
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
import requests


class ElasticSearch(Summary):

    def __init__(self, json_resp: dict, api_url: str = ''):
        super().__init__(plugin_name='elasticsearch',
                         template='elasticsearch_summary.j2')
        if not api_url:
            self.api_url = 'http://localhost:9200'
        else:
            self.api_url = api_url
        self.json_resp = json_resp

    @property
    def api_url(self) -> str:
        return self._api_url

    @api_url.setter
    def api_url(self, url: str) -> None:
        self._api_url = url

    @property
    def json_resp(self) -> defaultdict or list:
        return self._json_resp

    @json_resp.setter
    def json_resp(self, json_resp: dict or list) -> None:
        try:
            if type(json_resp) == list:
                self._json_resp = json_resp
            else:
                self._json_resp = defaultdict(None, json_resp)
        except Exception as e:
            self._json_resp = defaultdict(None, {'error': e})

    @property
    def version(self):
        try:
            return self.json_resp['version']['number']
        except KeyError:
            return None

    @classmethod
    def get(cls, api_url: str = ''):
        if not api_url:
            api_url = 'http://localhost:9200'
        headers = {'Accept': 'application/json; charset=UTF-8'}
        try:
            res = requests.get(url=api_url, headers=headers).json()
        except requests.ConnectionError:
            err = r"Fatal - ElasticSearch API seems to be down!"
            code = 500
            res = {'error': {'code': code, 'msg': err}}
        return cls(api_url=api_url, json_resp=res)

    @property
    def status(self) -> str:
        return self._status

    @status.setter
    def status(self, level: str) -> None:
        log_parser = LogParser()
        state = {
            'OK': log_parser.OK,
            'WARN': log_parser.WARN,
            'ERR': log_parser.ERROR
        }
        self._status = LogParser().set_log_level(state.get(level,
                                                           log_parser.WARN))

    def version_is_deprecated(self) -> bool:
        '''
        Validate version of ElasticSearch against stable versions.
        :return:
        '''
        versions_stable = ['6.0', '7.0']
        is_deprecated = False
        if not self.version:
            return True
        for key, ver in enumerate(versions_stable):
            ver_next = key + 0 if key >= len(versions_stable) else 1
            # Version is greater than 6.0 or 7.0
            if (self.version >= ver
                    or self.version >= versions_stable[ver_next]):
                break
            # Version is below than 6.0 and 7.0
            if self.version < ver and self.version < versions_stable[ver_next]:
                is_deprecated = True
                break

        return is_deprecated

    def summarize(self) -> None:
        tpl_init = TemplateRenderer(self.plugin_name, self.template)
        api_url = 'http://localhost:9200'
        result = ElasticSearch.get(api_url=api_url)
        self.summary['version_installed'] = result.version
        "Query Shards"
        api_url = api_url + '/_cat/shards'
        try:
            res_shards = ElasticSearch.get(api_url=api_url)
        except Exception:
            api_url = 'http://localhost:9200/_cluster/health'
            res_shards = ElasticSearch.get(api_url=api_url)

        count_shards = 0
        if type(res_shards.json_resp) == list:
            "Count shards"
            if res_shards.json_resp:
                for shard_dict in res_shards.json_resp:
                    if 'active_shards' in shard_dict and \
                       '/_cluster/health' in api_url:
                        count_shards = shard_dict['active_shards']
                        break
                    elif 'shard' in shard_dict:
                        count_shards += (1 if shard_dict['ip'] is not None
                                         else 0)

        tpl_str = tpl_init.template_render()
        # Print module header
        print(tpl_str.strip())

        err = ''
        self.status = 'OK'
        if self.version_is_deprecated():
            self.status = 'ERR'
            err = " \033[0;31m[DEPRECATED]\033[0m"
        output = "[%s] Version installed: %s%s\n" % (self.status,
                                                     result.version, err)
        self.status = 'WARN' if count_shards > 0 else 'OK'
        output += "[%s] Shards in use: %d\n" % (self.status, count_shards)
        print(output)


if __name__ == '__main__':
    res = ElasticSearch.get()
    res.summarize()
