#!/usr/bin/env python3
# -*- coding: future_fstrings -*-

"""
This module is used for querying and validating databases and database users
which are known to ISPConfig. All that is not known will be marked as WARN.
"""
import re
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.types.SQL import SQL


def get_server_id() -> int:
    server_id = 0
    with open('/usr/local/ispconfig/server/lib/config.inc.php', 'r') as \
            ro_stream:
        line = ro_stream.readline()
        pattern = r"^\$conf\['server_id'\]\s*=\s*\'([0-9]+)\';$"
        regex = re.compile(pattern)
        while line != '':
            match = regex.search(line)
            if match is not None:
                server_id = match.group(1)
                break
            line = ro_stream.readline()
    return int(server_id)


def get_mirror_status(server_id: int) -> (int, str):
    sql_query = f"""SELECT mirror_server_id FROM `server`
                    WHERE `server_id` = {server_id};"""
    mirror_id = SQL.db_query(sql_stmt=sql_query).result[0]['mirror_server_id']
    if mirror_id > 0:
        sql_query = f"""SELECT server_name FROM server
                        WHERE server_id = {mirror_id}"""
        mirrored_server = \
            SQL.db_query(sql_stmt=sql_query).result[0]['server_name']
    else:
        mirrored_server = None

    return mirror_id, mirrored_server


def get_master_server() -> str or None:
    master_server = None
    with open('/usr/local/ispconfig/server/lib/config.inc.php', 'r') as \
            ro_stream:
        line = ro_stream.readline()
        pattern = r"^\$conf\['dbmaster_host'\]\s*=\s*\'([a-z0-9-.]+)\';$"
        regex = re.compile(pattern)
        while line != '':
            match = regex.search(line)
            if match is not None:
                master_server = match.group(1)
                break
            line = ro_stream.readline()

    return master_server


def diff_databases() -> set:
    """
    Retrieve a diff of databases
    """
    server_id = get_server_id()
    sql_query = f"""SELECT `database_name`
    FROM `web_database`
    WHERE `server_id` = {int(server_id)}
    AND `database_name` != ''
    """
    db_ispc = SQL.db_query(sql_stmt=sql_query)
    db_ok = ('"grafana","rainloop","dbispconfig","phpmyadmin","sys",'
             '"information_schema","mysql","performance_schema"')
    sql_query = ("SELECT `schema_name` database_name FROM `schemata` WHERE "
                 f"`schema_name` NOT IN ({db_ok})")
    db_all = SQL.db_query(sql_stmt=sql_query, db='information_schema')

    diff = set()
    if (db_ispc.result is not None
            and db_all.result is not None):

        ispc_set = set()
        db_all_set = set()
        for ispc_dict in db_ispc.result:
            for db_all_dict in db_all.result:
                ispc_set.add(ispc_dict['database_name'])
                db_all_set.add(db_all_dict['database_name'])

        diff = db_all_set.symmetric_difference(ispc_set)

    return diff


def diff_database_users() -> set:
    """
    Retrieve a diff of database users
    """
    server_id = get_server_id()
    sql_query = f"""SELECT `database_user`
    FROM `web_database_user`
    WHERE `server_id` = {int(server_id)}
    OR `server_id` = 0"""
    db_users_ispc = SQL.db_query(sql_stmt=sql_query)
    db_users_ok = ('"ispconfig","root","prometheus","phpmyadmin","grafana",'
                   '"nagcheck","rainloop","debian-sys-maint","mariadb.sys",'
                   '"proxysql_monitor","mysql","mysql.session",'
                   '"mysql.infoschema","mysql.sys"')
    sql_query = "SELECT `user` from `user` where `user` like 'ispcsrv%';"
    multiserver_user = SQL.db_query(sql_stmt=sql_query, db='mysql')
    for userdict in multiserver_user.result:
        db_users_ok += f",\"{userdict['user']}\""

    sql_query = f"""SELECT `user` as database_user
    FROM `user`
    WHERE `user` NOT IN ({db_users_ok})"""
    db_users_all = SQL.db_query(sql_stmt=sql_query, db='mysql')

    diff = set()
    if (db_users_ispc is not None
            and db_users_all is not None):

        ispc_set = set()
        db_users_all_set = set()
        for ispc_dict in db_users_ispc.result:
            for db_users_all_dict in db_users_all.result:
                ispc_set.add(ispc_dict['database_user'])
                db_users_all_set.add(db_users_all_dict['database_user'])

        diff = db_users_all_set.difference(ispc_set)

    return diff


def summary() -> str:
    log_parser = LogParser()
    level = log_parser.set_log_level(log_parser.OK)
    db_diff = diff_databases()
    db_users_diff = diff_database_users()

    sum_str = ''

    if len(db_diff) > 0:
        level = log_parser.set_log_level(log_parser.ERROR)
        db_sum_str = (f"[{level}] Found databases which are "
                      "not created by ISPConfig!:\n"
                      "Make sure to fix these manually before migrating\n")
        for item in db_diff:
            db_sum_str += f"    \\_ {item}\n"
        sum_str = db_sum_str

    if len(db_users_diff) > 0:
        level = log_parser.set_log_level(log_parser.ERROR)
        db_users_sum_str = (f"[{level}] Found database users which are not "
                            "created by ISPConfig!\n"
                            "Make sure to fix these manually before migrating\n")
        for item in db_users_diff:
            db_users_sum_str += f"    \\_ {item}\n"
        sum_str += "\n" + db_users_sum_str

    sum_str = (f'[{level}] No issues found\n') if sum_str == '' else sum_str

    sub_mod = '--> \033[46mSubmodule: Database\033[0m\n'
    sum_str = sub_mod + sum_str

    return sum_str


if __name__ == '__main__':
    print(summary())
