#!/usr/bin/env python3
# -*- coding: future_fstrings -*-

"""
This module is used for detecting the ispconfig setup type
"""
from collections import defaultdict
from migrationsummary.ispconfig import database
from migrationsummary.ispconfig import ISPConfig
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.types import SQL
from socket import getfqdn


def is_multiserver() -> defaultdict:
    is_multi = False
    is_master = True
    is_mirror = False

    server_id = ISPConfig.SERVER_ID

    sql_query = """SELECT `server_id`
                   FROM `server`"""
    res = SQL.db_query(sql_stmt=sql_query)

    if 'error' in res.result:
        raise Exception(res.result)

    num_servers = len(res.result)
    master_server = database.get_master_server()

    if num_servers > 1:
        is_multi = True
        is_master = True
    elif master_server is not None:
        is_multi = True
        is_master = False
    elif num_servers == 1 and master_server is not None:
        is_multi = False
        is_master = True

    mirror_id, mirrored_server = database.get_mirror_status(server_id)
    if mirrored_server:
        is_mirror = True

    return defaultdict(None, {
        'is_master': is_master,
        'is_multi': is_multi,
        'is_mirror': is_mirror
    })


def summary() -> str:
    setup = is_multiserver()
    log_parser = LogParser()
    level = log_parser.set_log_level(log_parser.OK)

    sum_str = ""
    if setup['is_multi']:
        setup_type = "Multi"
        master_server = database.get_master_server()
        # If executed on a master server, get_master_Server returns None
        # should be safe as we only execute this if we already know that
        # we're on a multiserver setup
        if master_server is None:
            master_server = getfqdn()

        if setup['is_mirror']:
            setup_type = "Mirror"
            mirrored_server_id, mirrored_server = \
                database.get_mirror_status(ISPConfig.SERVER_ID)
            sum_str = (f"\n       Mirror of Server: {mirrored_server}")

        level = log_parser.set_log_level(log_parser.WARN)
        sum_str = (f"[{level}] Detected Setup: {setup_type} Server\n"
                   f"       Masterserver: {master_server}"
                   f"{sum_str}")
    else:
        sum_str = f'[{level}] Detected Setup: Single Server'

    sub_mod = '--> \033[46mSubmodule: Detect Setup\033[0m\n'
    sum_str = sub_mod + sum_str

    return sum_str


if __name__ == '__main__':
    print(summary())
