#!/usr/bin/env python3
# -*- coding: future_fstrings -*-

"""
This module is used for calculating disk usage of given paths and to make
a Recommendation for doing a second transfer as well as calculating
the estimated downtime for it
"""
from migrationsummary.ispconfig import ISPConfig
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.types import SQL
from os import popen, path
from types import GeneratorType


def get_doc_roots() -> GeneratorType:
    sql_stmt = (f"""SELECT `document_root`
    FROM `web_domain`
    WHERE `server_id` = {ISPConfig.SERVER_ID}
    AND (`document_root` IS NOT NULL
    OR `document_root` != '')""")
    for res in SQL.db_query(sql_stmt).result:
        if 'error' in res:
            yield ''
        if 'document_root' in res:
            yield res['document_root']


def calculate_disk_usage(dirpath: str) -> int:
    if not path.isdir(dirpath):
        return 0

    command = "du -s " + dirpath + " |cut -d$'\t' -f1"
    stream = popen(command)
    size = stream.read()
    stream.close()
    return size


def get_free_disk_space() -> int:
    command = "df --output=avail / | tail -n1"
    stream = popen(command)
    avail = int(stream.read())
    stream.close()
    return avail


def summary() -> str:
    paths = ['/var/lib/mysql']
    paths += ['/var/www']
    paths += ['/var/vmail']
    size_web = []
    size_sql = []
    size_mail = []
    log_parser = LogParser()
    disk_space = get_free_disk_space() / 1024

    for p in paths:
        if p == '/var/lib/mysql':
            [size_sql.append(calculate_disk_usage(p))]
        elif p == '/var/www':
            [size_web.append(calculate_disk_usage(p))]
        elif p == '/var/vmail':
            [size_mail.append(calculate_disk_usage(p))]

    usage_sql = 0.0
    usage_web = 0.0
    usage_mail = 0.0
    for size in size_sql:
        usage_sql += int(size)
    usage_sql = usage_sql / 1024
    for size in size_web:
        usage_web += int(size)
    usage_web = usage_web / 1024
    for size in size_mail:
        usage_mail += int(size)
    usage_mail = usage_mail / 1024
    est_downtime = (usage_web + usage_sql + usage_mail) / (65*60)
    hours, seconds = divmod(est_downtime * 60, 3600)
    minutes, seconds = divmod(seconds, 60)

    if disk_space < usage_sql:
        level = log_parser.set_log_level(log_parser.ERROR)
    else:
        level = log_parser.set_log_level(log_parser.OK)

    sum_str = (f"[{level}] Free Space: {disk_space / 1024:.4f} GB\n")
    level = log_parser.set_log_level(log_parser.INFO)
    sum_str += ((f"[{level}] Web-Files: {usage_web / 1024:.4f} GB\n"
                 f"[{level}] Database: {usage_sql / 1024:.4f} GB\n"
                 f"[{level}] Mail: {usage_mail / 1024:.4f} GB\n"))

    if est_downtime >= 30:
        level = log_parser.set_log_level(log_parser.WARN)
    else:
        level = log_parser.set_log_level(log_parser.OK)
    sum_str += (f'[{level}] Estimated copy time: {hours:02.0f}h'
                f' {minutes:02.0f}min {seconds:02.0f}s')
    return sum_str


if __name__ == '__main__':
    print(summary())
