#!/usr/bin/env python3
# -*- coding: utf8 -*-
from collections import defaultdict
from migrationsummary.models.Summary import Summary
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
from posix import listdir
from pymongo import MongoClient
from pymongo.errors import ConnectionFailure


class MongoDB(Summary):

    def __init__(self) -> None:
        super().__init__(plugin_name='mongodb', template='mongodb_summary.j2')

    @property
    def mongo_client(self) -> MongoClient:
        """mongo_client: readonly property

        Spawn new instance of MongoClient
        """
        config_params = {
            'connectTimeoutMS': 2000,
            'socketTimeoutMS': 2000,
            'serverSelectionTimeoutMS': 2000
        }

        return MongoClient(**config_params)

    @property
    def mongo_config(self) -> defaultdict or None:
        "Read mongodb config and return parameters as defaultdict"
        config = '/etc/mongodb.conf'
        try:
            with open(config, 'r') as stream:
                return defaultdict(
                    None, (line.strip().split('=', 1)
                           for line in stream.readlines()
                           if '=' in line and not line.startswith('#')))
        except FileNotFoundError:
            return None

    @property
    def server_running(self) -> bool:
        "readonly"
        try:
            self.mongo_client.admin.command('ismaster')
            return True
        except ConnectionFailure:
            return False

    @property
    def databases(self) -> list:
        "Determine if mongodb server holds any databases"
        db_list = []
        try:
            db_list = self.mongo_client.list_database_names()
        except ConnectionFailure:
            # Try to read databases from local db dir
            if self.mongo_config and 'dbpath' in self.mongo_config:
                db_list = [
                    db.strip('.ns').strip()
                    for db in listdir(self.mongo_config['dbpath'])
                    if db.endswith('ns')
                ]

        for key, item in enumerate(db_list[:]):
            # Ignore local database
            if item == 'local':
                db_list.pop(key)

        return db_list

    def __summary_set_server_running(self) -> None:
        log_parser = LogParser()
        self.summary['state'] = log_parser.set_log_level(
            log_parser.INFO
        )
        self.summary['is_running'] = self.server_running

    def __summary_set_db(self) -> None:
        log_parser = LogParser()
        self.summary['db'] = {}
        self.summary['db']['state'] = log_parser\
            .set_log_level(log_parser.OK)
        self.summary['db']['message'] = 'No databases found!'

        if self.databases and any(self.databases):
            self.summary['db']['db_list'] = self.databases
            self.summary['db']['state'] = log_parser\
                .set_log_level(log_parser.WARN)
            self.summary['db']['message'] = (
                'Please check! It seems that there are databases in use:'
                )
        if not self.mongo_config and not self.databases:
            self.summary['db']['state'] = log_parser\
                .set_log_level(log_parser.ERROR)
            self.summary['db']['message'] = (
                'Error retrieving database information!'
                ' The server is not running and no config could be found.')

    def summarize(self) -> None:
        self.__summary_set_server_running()
        self.__summary_set_db()
        tpl_init = TemplateRenderer(plugin_name=self.plugin_name,
                                    tpl_name=self.template)
        tpl_string = tpl_init.template_render(
            **{
                'mongo_config': self.summary
                })
        print(tpl_string)
