#!/usr/bin/env python3
# -*- coding: future_fstrings -*-
from posix import listdir
from subprocess import run, PIPE, STDOUT
from hashlib import md5
from json import load as json_parse
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
from migrationsummary.parsing.logging.LogParser import LogParser
from collections import defaultdict
from os import path
from migrationsummary.parsing.config.YamlParser import YamlParser
from migrationsummary.models.Summary import Summary


class Nginx(Summary):

    def __init__(self) -> None:
        self.plugin_name = 'nginx'
        self.summary = defaultdict()
        self.template = 'nginx_summary.j2'

    @property
    def base_path(self) -> str:
        return '/etc/nginx'

    @property
    def is_nginx(self) -> bool:
        possible_variants = ('apache2', 'nginx')
        cmds = (('service', sn, 'status') for sn in possible_variants)

        cmd_results = (run(cmd, stdout=PIPE, stderr=STDOUT) for cmd in cmds)
        is_nginx = False
        for result in cmd_results:
            if (result.returncode > 0
                    and 'apache2' in result.stdout.decode('utf8')):
                is_nginx = True
            if (result.returncode == 0
                    and 'nginx' in result.stdout.decode('utf8')):
                is_nginx = True
        return is_nginx

    def diff_dir_contents(self, yaml_file: str, p: str) -> defaultdict:
        """Diff contents of nginx subdirectories against list of allowed files
        Parameters:
        - yaml_file: The yaml file path including a list of allowed files
        - p: Part of nginx file path for e.g. 'conf.d' or 'cdn'
        """

        log_parser = LogParser()
        parser = YamlParser(yaml_file, 'r')
        raw_yaml = ''.join(parser.yaml_file.read())
        allowed_files = parser.parse(raw_yaml)

        # Check for valid key inside YAML
        key = 'files'
        if key not in allowed_files:
            raise Exception(f'No key "{key}" found inside YAML')

        # Make diff and validate
        d = path.join(self.base_path, p)
        if not path.isdir(d):
            return defaultdict(
                None, {
                    'path':
                    d,
                    'state': log_parser.set_log_level(log_parser.OK),
                    'files': ()
                })

        contents = set(str())
        {contents.add(path.join(d, f)) for f in listdir(d)}
        file_diff = list(contents - set(allowed_files[key]))

        return defaultdict(
            None, {
                'path':
                d,
                'state': (log_parser.set_log_level(
                    log_parser.WARN if any(file_diff) else log_parser.OK)),
                'files':
                file_diff
            })

    def md5_validate_nginx_conf(self) -> None:
        log_parser = LogParser()
        md5_hash = md5()
        config = path.join(self.base_path, 'nginx.conf')
        hashes_file = path.join(f'{self.module_path}'
                                '/conf/nginx_valid_md5.json')

        # Read file and build md5 hash
        with open(config, 'rb') as f:
            # Read and update hash in chunks of 4K
            for byte_block in iter(lambda: f.read(4096), b""):
                md5_hash.update(byte_block)
            md5sum = md5_hash.hexdigest()

        hashes = json_parse(open(hashes_file))

        self.summary['config_checksum'] = 'Invalid File Hash!'
        self.summary['config_checksum_state'] = log_parser.set_log_level(
            log_parser.ERROR)
        if md5sum in set(hashes):
            self.summary['config_checksum'] = md5sum
            self.summary['config_checksum_state'] = log_parser.set_log_level(
                log_parser.OK)

    def __summary_set_module_list(self, val: str) -> None:
        # Remove unwanted strings\
        log_parser = LogParser()
        str_unwanted = ('--with-', '--add-module=')
        if val.startswith('--add-module') or val.endswith('_module'):
            module = (val.replace(u, '') for u in str_unwanted if u in val)
            if 'modules' not in self.summary:
                self.summary['modules'] = []
            for m in module:
                # Create new dict key "modules" if not already present
                mod = {
                    'module':
                    m,
                    'state':
                    log_parser.set_log_level(log_parser.ERROR if 'geoip' in
                                             m else log_parser.OK)
                }
                # Append module dict to summary
                self.summary['modules'].append(mod)

    def __summary_set_version(self, val: str) -> None:
        log_parser = LogParser()
        if val.startswith('nginx/'):
            ngx_ver = val.strip('nginx/')
            self.summary['version'] = ngx_ver
            self.summary['version_state'] = log_parser\
                .set_log_level(log_parser.OK)
            if self.summary['version'] < '1.19.6':
                self.summary['version_state'] = log_parser\
                    .set_log_level(log_parser.ERROR)

    def __summary_set_file_diffs(self, data: list) -> None:
        # File diffs
        file_list = []
        for item in data:
            file_list.append(
                self.diff_dir_contents(item['allowed_files_conf'],
                                       item['config_dir']))
        self.summary['file_diffs'] = file_list

    def __summary_set_webserver_type(self) -> None:
        log_parser = LogParser()
        self.summary['is_nginx'] = self.is_nginx
        self.summary['webserver_state'] = (log_parser.set_log_level(
            log_parser.ERROR if not self.is_nginx else log_parser.OK))

    def summarize(self) -> None:
        self.__summary_set_webserver_type()
        if self.is_nginx:
            cmd = ('nginx', '-V')
            result = run(cmd, stdout=PIPE, stderr=STDOUT, check=True)
            ngx_data = set(str())
            if result.returncode == 0:
                awk_filter = ('awk', '-F', ':', '/1/ {print $2}')
                result_filtered = run(awk_filter,
                                      input=result.stdout,
                                      stdout=PIPE,
                                      stderr=STDOUT)
                if result_filtered.returncode == 0:
                    output = result_filtered.stdout.decode('utf8')
                    ngx_data = set(output.strip().lstrip().split())
                    # Filter out unnecessary data
                    # from nginx compile args output
                    ngx_filter = set(
                        str(lambda: ngx_filter.add(i) for i in ngx_data
                            if not i.startswith(('--add-module', 'nginx/'))
                            and not i.endswith('_module')))
                    ngx_data = ngx_data - ngx_filter

            # Prepare nginx summary
            if any(ngx_data):
                for v in ngx_data:
                    self.__summary_set_version(v)
                    self.__summary_set_module_list(v)

            # Get md5 hash of nginx.conf
            self.md5_validate_nginx_conf()

            # Get file diffs
            scan_data = list([{
                'allowed_files_conf':
                f'{self.module_path}/conf/nginx_cdn.yml',
                'config_dir':
                'cdn'
            }, {
                'allowed_files_conf':
                f'{self.module_path}/conf/nginx_conf-d.yml',
                'config_dir':
                'conf.d'
            }, {
                'allowed_files_conf':
                f'{self.module_path}/conf/nginx_modules-enabled.yml',
                'config_dir':
                'modules-enabled'
            }])
            self.__summary_set_file_diffs(scan_data)

        # Render template
        tpl_init = TemplateRenderer(self.plugin_name, self.template)
        tpl_str = tpl_init.template_render(**{'nginx_config': self.summary})
        print(tpl_str)
