#!/usr/bin/env python3
# -*- coding: future_fstrings -*-
from collections import defaultdict
import migrationsummary.funcs as func
from migrationsummary.ispconfig import ISPConfig
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
from migrationsummary.parsing.config.YamlParser import YamlParser
from migrationsummary.models.Summary import Summary
from os import listdir
from os import path
import pymysql as my
from pymysql.cursors import DictCursor
from shlex import quote
from subprocess import run, PIPE
from typing import Generator, Any


class PrintOnce:
    def __init__(self, data):
        self.data = data
        self.printed = False

    def __str__(self):
        if not self.printed:
            self.printed = True
            return self.data
        return ''


class Php(Summary):
    def __init__(self) -> None:
        super().__init__(plugin_name='php', template='php_summary.j2')

    @property
    def db_conf(self) -> Any:
        sql_conf = '/usr/local/ispconfig/server/lib/mysql_clientdb.conf'
        if not path.isfile(sql_conf):
            return None

        # Read mysql_clientdb.conf line by line
        try:
            config = []
            with open(sql_conf, 'r') as ro_stream:
                line = ro_stream.readline()
                while line != '':
                    # Skip php tags
                    if (line.startswith('<?php')
                            or line.startswith('?>')):
                        line = ''

                    # Strip "$" from line
                    if line.startswith('$'):
                        line = line[1:]

                    # Trim unnecessary content
                    line = func.trim(line)

                    # Strip ";" from line
                    if line and line.endswith(';'):
                        line = line[:0] + line[:len(line) - 1]

                    # Append to config
                    if line:
                        config.append(line.split('='))
                    # Read next line
                    line = ro_stream.readline()
            ro_stream.close()

            # Create config dictionary
            config_dict = {}
            if config:
                for arr in config:
                    # Dict Comprehension to create key/value pair from arr
                    part_dict = {arr[i]: arr[i + 1]
                                 for i in range(0, len(arr), 2)}
                    # Merge part_dict into config_dict
                    config_dict = {**config_dict, **part_dict}
            # All done now return
            return my.connect(host=config_dict['clientdb_host'],
                              user=config_dict['clientdb_user'],
                              password=config_dict['clientdb_password'],
                              database='dbispconfig',
                              cursorclass=DictCursor)
        except Exception:
            return None

    @property
    def fetch_from_db(self) -> Generator[dict, None, None]:
        if self.db_conf:
            cursor = self.db_conf.cursor()
            cursor.execute(
                "SELECT DISTINCT `php_version`, `php_fpm_pool_dir` "
                "FROM `server_php` "
                f"WHERE `server_id` = {ISPConfig.SERVER_ID} "
                "AND `php_version` != ''")
            data = cursor.fetchall()
            if data and any(data):
                for item in data:
                    yield item

    # TODO: move to helper class with function from NGINX class
    def diff_dir_contents(self, yaml_file: str, p: str) -> defaultdict:
        """Diff contents of php subdirectories against list of allowed files
        Parameters:
        - yaml_file: The yaml file path including a list of allowed files
        - p: Part of php file path for e.g. 'mods-available'
        """

        log_parser = LogParser()
        parser = YamlParser(yaml_file, 'r')
        raw_yaml = ''.join(parser.yaml_file.read())
        allowed_files = parser.parse(raw_yaml)

        # Check for valid key inside YAML
        key = 'files'
        if key not in allowed_files:
            raise Exception(f'No key "{key}" found inside YAML')

        if not path.isdir(p):
            return defaultdict(
                None, {
                    'path':
                    p,
                    'state': log_parser.set_log_level(log_parser.OK),
                    'files': ()
                })

        # Make diff and validate
        contents = set(str())
        {contents.add(f) for f in listdir(p)}
        file_diff = list(contents - set(allowed_files[key]))

        return defaultdict(
            None, {
                'path':
                p,
                'state': (log_parser.set_log_level(
                    log_parser.WARN if any(file_diff) else log_parser.OK)),
                'files':
                file_diff
            })

    def __summary_set_version(self, version: str):
        log_parser = LogParser()
        self.summary['is_deprecated'] = False
        self.summary['state'] = log_parser\
            .set_log_level(log_parser.INFO)
        if version < '7.0':
            self.summary['is_deprecated'] = True
        self.summary['version'] = version

    def __summary_set_file_diffs(self, data: list) -> None:
        # File diffs
        file_list = []
        for item in data:
            file_list.append(
                self.diff_dir_contents(item['allowed_files_conf'],
                                       item['config_dir']))
        self.summary['file_diffs'] = file_list

    def __summary_set_fpm_pools(self, pool_path: str):
        fpm_pools = []
        # Append each fpm pool to fpm_pools list
        fpm_pools = [path.join(pool_path, file) for file in listdir(pool_path)]
        self.summary['fpm_pools'] = [fpm_pool for fpm_pool in fpm_pools]

    def __summary_set_module_variances(self, module_list_is: set,
                                       module_list_should: list):
        self.summary['module_variances'] = (set(module_list_is) -
                                            set(module_list_should))

    def __get_module_list(self, fpm_pool_path: str):
        bin_path = path.join(
            fpm_pool_path[:len(fpm_pool_path) - len('etc/pool.d')].strip(),
            'bin/php')
        cmd = (quote(bin_path), '--modules')
        cmd_result = run(cmd, check=True, stdout=PIPE, stderr=PIPE)
        # Trim headings like [PHP Modules]
        result_grep = ('grep', '-v', '\\[\\(.*\\)\\]')
        result_filtered = run(result_grep,
                              input=cmd_result.stdout,
                              stdout=PIPE,
                              stderr=PIPE)
        # Convert module output string to list
        result = result_filtered.stdout.decode('utf8').split('\n')
        # Remove duplicate values from module list
        module_list = list(defaultdict().fromkeys(set(result)))
        module_list.remove('')
        # Remove unnecessary values from module list
        module_list[:] = (v for v in module_list
                          if not v.startswith('the ionCube'))
        return module_list

    def summarize(self) -> None:
        log_parser = LogParser()
        config = path.join(f'{self.module_path}/conf', 'php_config.yml')
        yaml_parser = YamlParser(config, 'r')
        data = ''.join(yaml_parser.yaml_file.read())
        file_content = yaml_parser.parse(data)
        versions_data = file_content['php']
        summaries = []

        for item in self.fetch_from_db:
            if path.isdir(item['php_fpm_pool_dir']):
                self.summary = defaultdict()
                if item['php_version']:
                    self.__summary_set_version(item['php_version'])
                if item['php_fpm_pool_dir']:
                    self.__summary_set_fpm_pools(item['php_fpm_pool_dir'])
                    module_list = \
                        self.__get_module_list(item['php_fpm_pool_dir'])
                if (self.summary['is_deprecated']
                        and any(self.summary['fpm_pools'])):
                    self.summary['state'] = log_parser\
                        .set_log_level(log_parser.ERROR)
                for version in versions_data:
                    if (version['version'] in self.summary['version']
                            and self.summary['version'] >= '5.6'):
                        self.__summary_set_module_variances(
                            module_list, version['modules'])
                        # Get file diffs
                        scan_data = list([{
                            'allowed_files_conf':
                            f'{self.module_path}/conf/php_mods-available.yml',
                            'config_dir':
                            f"/opt/php-{version['version']}/lib/mods-available"
                        }])
                        self.__summary_set_file_diffs(scan_data)

                if ('module_variances' in self.summary
                        and any(self.summary['module_variances'])):
                    self.summary['state'] = log_parser\
                        .set_log_level(log_parser.WARN)
                summaries.append(self.summary)

        tpl_init = TemplateRenderer(self.plugin_name, self.template)
        tpl_str = tpl_init.template_render(
            **{
                'summaries':
                summaries,
                'deprecated_txt':
                PrintOnce('Deprecated versions installed:'),
                'versions_modules_txt':
                PrintOnce('Most recent versions and module variances:')
            })

        print(tpl_str)
