#!/usr/bin/env python3
# -*- coding: future_fstrings -*-
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.models.Summary import Summary
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
from migrationsummary.types import SQL
import dns.resolver
import tldextract


class VhostSummary(Summary):

    def __init__(self):
        super().__init__(plugin_name='vhostsummary',
                         template='vhost_summary.j2')
        self.snippet_list = {}

    def getDirectiveSnippetName(self, id):
        """Returns the name of a directive snippet.

        Parameters
        ----------
        id
            id of directive snippet in ispconfig database.

        Returns
        -------
            query result containing name of snippet.

        """
        stmt = (f"""select name
                    from directive_snippets
                    where directive_snippets_id = '{id}';""")
        query = SQL.db_query(stmt)
        return query.result

    def getDirectiveSnippets(self, id, folder):
        """Generates output for directive Snippets.

        Parameters
        ----------
        id :
            ID of main vhost directive snippet.
        folder :
            Subfolder directive snippets id list.
            Should look like "subfolder/:<ID>:0" in several
            lines

        Returns
        -------
        type
            Log String for Directive Snippets.

        """
        log_parser = LogParser()
        loglevel = log_parser.set_log_level(log_parser.INFO)

        if id == 0:
            ret = (f"    \\__ [{loglevel}] Snippet: "
                   "Custom configuration / No Snippet\n")
        else:
            name = self.getDirectiveSnippetName(id)
            if len(name) == 0:
                loglevel = log_parser.set_log_level(log_parser.WARN)
                ret = (f"    \\__ [{loglevel}] invalid Snippet ID: {id} "
                       "please check configuration\n")
            else:
                name_str = name[0]["name"]
                if name_str in self.snippet_list:
                    self.snippet_list[name_str] += 1
                else:
                    self.snippet_list[name_str] = 1
                ret = f"    \\__ [{loglevel}] Snippet: {name_str}\n"
        # Split Subfolders and get snippets
        if folder != "":
            subfolders = folder.splitlines()
            for subfolder in subfolders:
                subfolder = subfolder.split(":")
                snippetid = subfolder[1]
                folder = subfolder[0]
                if snippetid == 0:
                    loglevel = log_parser.set_log_level(log_parser.WARN)
                    ret += (f"    \\__ [{loglevel}] Subfolder: {folder} "
                            " uses Snippet ID: 0, this should not be possible,"
                            " please check configuration\n")
                    continue

                subfolder_snippet = \
                    self.getDirectiveSnippetName(snippetid)
                if len(subfolder_snippet) == 0:
                    loglevel = log_parser.set_log_level(log_parser.WARN)
                    ret += (f"        \\__ [{loglevel}] Subfolder: {folder}, "
                            f"invalid Snippet ID: {snippetid} "
                            "please check configuration\n")
                    continue

                name_str = subfolder_snippet[0]["name"]
                ret += (f"        \\__ [{loglevel}] Subfolder: {folder},"
                        f" Directive: {name_str}\n")
                if name_str in self.snippet_list:
                    self.snippet_list[name_str] += 1
                else:
                    self.snippet_list[name_str] = 1
        return ret

    def getVhostList(self):
        """Gets a vhost list from ispconfig database.

        Returns
        -------
        type
            query result containing
                            domain_id,
                            domain,
                            folder_directive_snippets,
                            directive_snippets_id

        """
        stmt = ("""select domain_id, domain, folder_directive_snippets,
                    directive_snippets_id from web_domain
                    where `type` like 'vhost%';""")
        query = SQL.db_query(stmt)

        return query.result

    def getDNSInformation(self, domain):
        """Gets Information about the NS of a domain.

        Parameters
        ----------
        domain
            domain name, can be a subdomain as well

        Returns
        -------
            Log output

        """
        log_parser = LogParser()
        message = ""
        loglevel = ""
        try:
            tsd, td, tsu = tldextract.extract(domain)
            domain = td + '.' + tsu
            query = dns.resolver.query(domain, 'NS')
            ns = str(query[0].target)
            if "timmehosting" in ns:
                loglevel = log_parser.set_log_level(log_parser.OK)
                message = "TimmeHosting Nameserver"
            else:
                loglevel = log_parser.set_log_level(log_parser.WARN)
                message = "Other Nameserver"
        except Exception as e:
            print(e)
            query = ""
        if query == "":
            loglevel = log_parser.set_log_level(log_parser.WARN)
            message = "No Nameserver"
        ret = f"    \\__ [{loglevel}] DNS: {message}\n"
        return ret

    def summarize(self) -> None:
        tpl_init = TemplateRenderer(self.plugin_name, self.template)
        tpl_str = tpl_init.template_render()
        print(tpl_str.strip())
        vhosts = self.getVhostList()
        pre_str = ""
        out_str = ""
        log_parser = LogParser()
        for host in vhosts:
            out_str += (f"--> vHost ID: {host['domain_id']},\t"
                        f"Domain: {host['domain']}\n")
            if host["domain"]:
                directive_snippet = self.getDirectiveSnippets(
                    host["directive_snippets_id"],
                    host["folder_directive_snippets"])
                out_str += directive_snippet
                out_str += self.getDNSInformation(host["domain"])
            else:
                loglevel = log_parser.set_log_level(log_parser.ERROR)
                message = ("incompletely deleted Domain, please fix manually"
                           " by deleting DB Entry with the command:\n"
                           "              mysql"
                           " --defaults-extra-file=/etc/mysql/debian.cnf"
                           " dbispconfig -e 'DELETE FROM `web_domain` WHERE"
                           f" `domain_id` = {host['domain_id']}'")
                out_str += f"    \\__ [{loglevel}]: {message}\n"
            out_str += "\n"
        if out_str != "":
            pre_str = "\nSnippet Overview:\n=====================\n"
        for i in self.snippet_list:
            pre_str += "Snippet: "
            pre_str += str(i)
            pre_str += ", used "
            pre_str += str(self.snippet_list[i])
            pre_str += " times\n"
        print(pre_str)
        print(out_str)


if __name__ == '__main__':
    None
