#!/usr/bin/env python3
# -*- coding: future_fstrings -*-
from collections import defaultdict
from migrationsummary.ispconfig import ISPConfig
from migrationsummary.parsing.logging.LogParser import LogParser
from migrationsummary.types import SQL
from migrationsummary.models.Summary import Summary
from migrationsummary.parsing.template.TemplateRenderer import TemplateRenderer
from typing import Generator


class WebStats(Summary):

    def __init__(self):
        super().__init__(plugin_name='Webstats',
                         template='webstats_summary.j2')

    def query_all_active(self) -> defaultdict or list:
        stmt = f"""SELECT `domain`, `stats_type` FROM `web_domain` WHERE
                `active` = 'y' AND `server_id` = {ISPConfig.SERVER_ID}
                AND `stats_type` IN ('webalizer', 'awstats', 'goaccess')"""
        query = SQL.db_query(stmt)

        return query.result

    def process_results(self) -> None or Generator:
        '''
        Process database results and returns a Generator object which
        contains the site name, stats type as well as status (OK, ERR).
        If no data could be found "None" will be returned.

        :returns None or Generator
        '''
        results = self.query_all_active()
        log_parser = LogParser()
        if 'error' in results:
            print(results)

        if not results:
            return None

        if any(results):
            for key, value in enumerate(results):
                dom_res = defaultdict(dict,
                                      {'site': value['domain'],
                                       'stats_type': value['stats_type'],
                                       'status': log_parser.set_log_level(
                                           log_parser.ERROR
                                           if value['stats_type'] != 'goaccess'
                                           else log_parser.OK)})
                yield dom_res
        return

    def summarize(self) -> None:
        tpl_init = TemplateRenderer(self.plugin_name, self.template)
        tpl_str = tpl_init.template_render()
        data = self.process_results()

        # Print plugin header
        print(tpl_str.strip())

        num_printed = 0
        for dom_res in data:
            output = (f"[{dom_res['status']}] {dom_res['site']}:"
                      f"\\_ Uses: {dom_res['stats_type']}")
            print(output)
            num_printed += 1
        # No data found
        if not num_printed:
            print(r"[--] Empty data!")
        return


if __name__ == '__main__':
    init = WebStats()
    init.summarize()
